-- GENERATED FILE, DO NOT MODIFY!
-- $Header: /code/pg_trompe/schema.sql,v 1.1 2009/01/09 23:14:49 doj Exp $

/** SCHEMA replication
Contains all functions and tables used for pg_trompe replication.
*/
CREATE SCHEMA replication;
COMMENT ON SCHEMA replication IS E'Contains all functions and tables used for pg_trompe replication.';
-- $Header: /code/pg_trompe/functions.sql,v 1.1 2009/01/09 23:14:48 doj Exp $

/** FUNCTION replication.nodes_insert()

Trigger function which writes INSERT rows into the replication.log
table for all replicated tables when a new node is added. The function
is called by an insert trigger on table replication.nodes.
*/
CREATE OR REPLACE FUNCTION replication.nodes_insert()
RETURNS trigger
AS $emp_stamp$
 DECLARE
  t RECORD;
 BEGIN
  FOR t IN select * from replication.tables order by nr LOOP

    IF t.lo = TRUE THEN
      EXECUTE 'insert into replication.log(node, tablename, action, nr, lo)
               select ''' || NEW.num || ''' , ''' || t.name || ''' , ''INSERT'' , nr , lo
               from ' || t.name;
    ELSE
      EXECUTE 'insert into replication.log(node, tablename, action, nr)
               select ''' || NEW.num || ''' , ''' || t.name || ''' , ''INSERT'' , nr
               from ' || t.name;
    END IF;

  END LOOP;
  RETURN NEW;
 END;
$emp_stamp$ LANGUAGE plpgsql;

/** FUNCTION replication.newevent(tablename_ text, action_ text, nr_ int8, lo_ int8)

Inserts a new event into replication.log for all neighbour nodes.

@param tablename_ table name which contains row

@param action_ event type. See TABLE replication.log for valid values.

@param nr_ row identifier or NULL

@param lo_ large object OID or NULL

*/
CREATE OR REPLACE FUNCTION replication.newevent(tablename_ text, action_ text, nr_ int8, lo_ int8)
RETURNS int
AS $emp_stamp$
 BEGIN
  insert into replication.log(node,tablename,action,nr,lo)
  select num, tablename_, action_, nr_, lo_
  from replication.nodes
  where num not in (select num from replication.nodes_skip);
  RETURN 0;
 END;
$emp_stamp$ LANGUAGE plpgsql;

/** FUNCTION replication.tableevent()
Trigger function on replicated table without large object (lo) column which creates entries in replication.log.
*/
CREATE OR REPLACE FUNCTION replication.tableevent()
RETURNS trigger
AS $emp_stamp$
 DECLARE
  tname		text := TG_TABLE_SCHEMA || '.' || TG_TABLE_NAME;
 BEGIN
  IF TG_OP = 'INSERT' THEN
    perform replication.newevent(tname, TG_OP, NEW.nr, NULL);
    RETURN NEW;

  ELSIF TG_OP = 'UPDATE' THEN
    IF NEW.nr <> OLD.nr THEN
      RAISE EXCEPTION 'Altering PK is forbidden in replicated table % (% -> %)', tname, OLD.nr, NEW.nr;
      RETURN NULL;
--      perform replication.newevent(tname, 'DELETE', OLD.nr, NULL);
--      perform replication.newevent(tname, 'INSERT', NEW.nr, NULL);
    ELSE
      delete from replication.log where tablename=tname and action=TG_OP and nr=NEW.nr;
      perform replication.newevent(tname, TG_OP, NEW.nr, NULL);
    END IF;
    RETURN NEW;

  ELSIF TG_OP = 'DELETE' THEN
    delete from replication.log where tablename=tname and nr=OLD.nr;
    perform replication.newevent(tname, TG_OP, OLD.nr, NULL);
    RETURN OLD;
  END IF;

  RETURN NULL;
 END;
$emp_stamp$ LANGUAGE plpgsql;

/** FUNCTION replication.loevent()
Trigger function on replicated table with large object (lo) column which creates entries in replication.log.
*/
CREATE OR REPLACE FUNCTION replication.loevent()
RETURNS trigger
AS $emp_stamp$
 DECLARE
  tname		text := TG_TABLE_SCHEMA || '.' || TG_TABLE_NAME;
 BEGIN
  IF TG_OP = 'INSERT' THEN
    perform replication.newevent(tname, TG_OP, NEW.nr, NEW.lo::int8);
    RETURN NEW;

  ELSIF TG_OP = 'UPDATE' THEN
    IF NEW.nr <> OLD.nr THEN
      RAISE EXCEPTION 'Altering PK is forbidden in replicated table % (% -> %)', tname, OLD.nr, NEW.nr;
      RETURN NULL;
    END IF;
    IF OLD.lo IS NULL and NEW.lo IS NULL THEN
      delete from replication.log where tablename=tname and action=TG_OP and nr=NEW.nr;
      perform replication.newevent(tname, TG_OP, NEW.nr, NULL);
    ELSEIF OLD.lo IS NULL THEN
      perform replication.newevent(tname, 'INSERT', NEW.nr, NEW.lo::int8);
    ELSEIF NEW.lo IS NULL THEN
      perform lo_unlink(OLD.lo);
      perform replication.newevent(tname, 'DELETE', OLD.nr, OLD.lo::int8);
    ELSIF NEW.lo <> OLD.lo THEN
      perform lo_unlink(OLD.lo);
      perform replication.newevent(tname, 'DELETE', OLD.nr, OLD.lo::int8);
      perform replication.newevent(tname, 'INSERT', NEW.nr, NEW.lo::int8);
    ELSE
      delete from replication.log where tablename=tname and action=TG_OP and nr=NEW.nr;
      perform replication.newevent(tname, TG_OP, NEW.nr, NEW.lo::int8);
    END IF;

    RETURN NEW;

  ELSIF TG_OP = 'DELETE' THEN
    delete from replication.log where tablename=tname and nr=OLD.nr;
    perform lo_unlink(OLD.lo);
    perform replication.newevent(tname, TG_OP, OLD.nr, OLD.lo::int8);

    RETURN OLD;

  END IF;

  RETURN NULL;
 END;
$emp_stamp$ LANGUAGE plpgsql;

/** FUNCTION replication.setseq(tablename text)
Sets the "nr" sequence on "tablename" to a new value which will allow the table to be used in replication.

@return -1 upon error, 1 upon success

*/
CREATE OR REPLACE FUNCTION replication.setseq(tablename text)
RETURNS int
AS $emp_stamp$
 DECLARE
  node_num text;
  maxnodes text;
  seq text;
 BEGIN
  -- check if replication is set up correctly
  SELECT val INTO node_num from replication.config where key = 'node_num';
  IF NOT FOUND THEN
    RAISE EXCEPTION 'replication node number not set';
    RETURN -1;
  END IF;

  -- get maxnodes
  SELECT val INTO maxnodes from replication.config where key = 'maxnodes';
  IF NOT FOUND THEN
    RAISE EXCEPTION 'replication maxnodes not set';
    RETURN -1;
  END IF;

  -- get new PK value. Because we can not SELECT INTO in conjuction
  -- with EXECUTE we use the replication.config table here with our
  -- temporary key 'DOJ'
  EXECUTE 'insert into replication.config(key,val)
  select ''DOJ'', COALESCE(max(nr)+' || maxnodes || ',' || node_num || ')
  from ' || tablename || '
  where (nr % ' || maxnodes || ') = ' || node_num;

  SELECT val INTO seq from replication.config where key = 'DOJ';
  IF NOT FOUND THEN
    RAISE EXCEPTION 'Error when inserting special DOJ config';
    RETURN -1;
  END IF;
  delete from replication.config where key = 'DOJ';

  -- set sequence value
  EXECUTE 'ALTER SEQUENCE ' || tablename || '_nr_seq
  INCREMENT BY ' || maxnodes || '
  NO MINVALUE NO MAXVALUE
  RESTART WITH ' || seq || '
  NO CYCLE;';

  PERFORM setval(tablename || '_nr_seq', seq::int8, false);

  RETURN 1;
 END;
$emp_stamp$ LANGUAGE plpgsql;

/** FUNCTION replication.setallseq()
Sets "nr" sequences for all replicated tables.

@return 0 upon success

*/
CREATE OR REPLACE FUNCTION replication.setallseq()
RETURNS int
AS $emp_stamp$
 DECLARE
  t RECORD;
 BEGIN
  FOR t IN select * from replication.tables LOOP
    PERFORM replication.setseq(t.name);
  END LOOP;
  RETURN 0;
 END;
$emp_stamp$ LANGUAGE plpgsql;

/** FUNCTION replication._register(tablename text)
Internal function to add a table to replication.

@return tablename with schema upon success, NULL upon error

*/
CREATE OR REPLACE FUNCTION replication._register(_tablename_ text)
RETURNS text
AS $emp_stamp$
 DECLARE
  c int;
  tablename_ text := _tablename_;
 BEGIN
  -- add schema to tablename_
  IF NOT tablename_ ~ E'\\.' THEN
    tablename_ := 'public.' || tablename_;
  END IF;

  -- check if table is not already handled
  SELECT count(*) INTO c from replication.tables where name = tablename_;
  IF c > 0 THEN
    RAISE EXCEPTION 'table % already in replication', tablename_;
    RETURN NULL;
  END IF;

  -- check if table exists
  SELECT count(*) INTO c from pg_catalog.pg_tables where tablename_ = schemaname || '.' || tablename;
  IF c = 0 THEN
    RAISE EXCEPTION 'table % does not exist', tablename_;
    RETURN NULL;
  END IF;

  -- register table
  insert into replication.tables(name) values(tablename_);

  SELECT INTO c replication.setseq(tablename_);
  IF c < 0 THEN
    RAISE EXCEPTION 'could not set sequence on table %', tablename_;
    RETURN NULL;
  END IF;

  RETURN tablename_;
 END;
$emp_stamp$ LANGUAGE plpgsql;

/** FUNCTION replication.register(tablename text)
Register a table for replication.

@param tablename name of the table. You may have to provide the schema name if the table is not in the 'public' schema.

@return tablename (full qualified) upon success, NULL upon error

*/
CREATE OR REPLACE FUNCTION replication.register(_tablename_ text)
RETURNS text
AS $emp_stamp$
 DECLARE
  tablename text;
  triggername text;
 BEGIN
  -- register the table
  SELECT INTO tablename replication._register(_tablename_);
  IF tablename is null THEN
    RETURN NULL;
  END IF;

  triggername := replace(tablename, '.', '_');

  -- create event trigger on table
  EXECUTE 'CREATE TRIGGER ' || triggername || '_replication_event
           BEFORE INSERT or UPDATE or DELETE
           ON ' || tablename || '
           FOR EACH ROW EXECUTE PROCEDURE replication.tableevent()';

  RETURN tablename;
 END;
$emp_stamp$ LANGUAGE plpgsql;

/** FUNCTION replication.registerlo(tablename text)
Register a table with "lo" column to manage large objects for replication.

@param tablename name of the table. You may have to provide the schema name if the table is not in the 'public' schema.

@return tablename (full qualified) upon success, NULL upon error

*/
CREATE OR REPLACE FUNCTION replication.registerlo(_tablename_ text)
RETURNS text
AS $emp_stamp$
 DECLARE
  tablename text;
  triggername text;
 BEGIN
  -- register the table
  SELECT INTO tablename replication._register(_tablename_);
  IF tablename is null THEN
    RETURN NULL;
  END IF;

  UPDATE replication.tables SET lo=TRUE WHERE name=tablename;

  triggername := replace(tablename, '.', '_');

  -- create event trigger to large object on table
  EXECUTE 'CREATE TRIGGER ' || triggername || '_replication_loevent
           BEFORE INSERT or UPDATE or DELETE
           ON ' || tablename || '
           FOR EACH ROW EXECUTE PROCEDURE replication.loevent()';

  RETURN tablename;
 END;
$emp_stamp$ LANGUAGE plpgsql;

/** FUNCTION replication.node_num(num_ int, d text)
Configure this database for replication.

@param num_ node number of this database

@param d description string of this node

@return num_ upon success, -1 upon error

*/
CREATE OR REPLACE FUNCTION replication.node_num(num_ int, d text)
RETURNS int
AS $emp_stamp$
 DECLARE
  n int;
  maxnodes int;
 BEGIN
  -- get maxnodes
  SELECT val::int INTO maxnodes from replication.config where key = 'maxnodes';

  -- check node number
  IF num_ < 0 or num_ >= maxnodes THEN
    RAISE NOTICE 'Node Num % is outside of configured range [0..%)', num_, maxnodes;
    RETURN -1;
  END IF;

  -- check if already configured
  SELECT val::int INTO n from replication.config where key='node_num';
  IF FOUND THEN
    RAISE NOTICE 'This Node is already configured as #%', n;
    RETURN -1;
  END IF;

  insert into replication.config(key,val,description) values('node_num', num_, 'Die Node Nummber dieses Nodes.');
  insert into replication.config(key,val,description) values('node_desc', d, 'Beschreibung dieses Nodes.');
  insert into replication.nodes_skip(num) values(num_);
--  delete from replication.log where node=num_;

  RETURN num_;
 END;
$emp_stamp$ LANGUAGE plpgsql;

/** FUNCTION replication.drop_node(n int)
Drop a node from replication.

@param n node number.

@return n upon success, -1 upon error.

*/
CREATE OR REPLACE FUNCTION replication.drop_node(n int)
RETURNS int
AS $emp_stamp$
 BEGIN
  -- we don't need to skip n any longer
  delete from replication.nodes_skip where num=n;

  -- check if we are n
  PERFORM * from replication.config where key = 'node_num' and val=n::text;
  IF FOUND THEN
    delete from replication.nodes;
    RETURN n;
  END IF;

  -- drop a neighbour node
  delete from replication.nodes where num=n;
  IF FOUND THEN
    RETURN n;
  END IF;

  -- nothing found
  RAISE NOTICE 'Node % was not included in replication.', n;
  RETURN -1;
 END;
$emp_stamp$ LANGUAGE plpgsql;

/** FUNCTION md5(id oid)

Calculates the MD5 sum of a large object. If no large object is found
the MD5 sum of the string value of id is calculated.

@param id OID of large object

@return MD5 sum

*/
CREATE OR REPLACE FUNCTION md5(id oid)
RETURNS text
AS $$
 DECLARE
  fd        integer;
  size      integer;
  c         int8;
  hashval   text;
  INV_READ  constant integer := x'40000'::integer; -- from libpq-fs.h
  SEEK_SET  constant integer := 0;
  SEEK_END  constant integer := 2;
 BEGIN

  SELECT loid INTO c FROM pg_catalog.pg_largeobject WHERE loid=id limit 1;
  IF NOT FOUND THEN
    RAISE NOTICE 'no large object % found, returning md5 of string', id;
    RETURN md5(id::text);
  END IF;

  fd   := lo_open(id, INV_READ);
  size := lo_lseek(fd, 0, SEEK_END);
  PERFORM lo_lseek(fd, 0, SEEK_SET);
  hashval := md5(loread(fd, size));
  PERFORM lo_close(fd);
  RETURN hashval;
 END;
$$
LANGUAGE plpgsql stable strict;

/** FUNCTION replication.ddl(t text)
Executes and replicates DDL commands.

@param t string containing DDL

@return 0 upon success, -1 upon error

*/
CREATE OR REPLACE FUNCTION replication.ddl(t text)
RETURNS int
AS $$
 DECLARE
  r int;
 BEGIN
  SELECT INTO r replication.newevent(t, 'DDL', NULL, NULL);
  IF r = 0 THEN
    EXECUTE t;
  END IF;
  RETURN r;
 END;
$$
LANGUAGE plpgsql strict;

/** FUNCTION replication.start(remote_num int)
Starts a replication with a neighbour node. This function is called by replication.pl.

@param remote_num neighbour node number

@return 0 upon success, -1 upon error

*/
CREATE OR REPLACE FUNCTION replication.start(remote_num int)
RETURNS int
AS $$
 DECLARE
  n int;
 BEGIN
  SELECT INTO n val::int from replication.config where key='node_num';
  IF NOT FOUND THEN
    RAISE EXCEPTION 'Replication has not been set up on this node.';
    RETURN -1;
  END IF;
  IF n = remote_num THEN
    RAISE EXCEPTION 'You can''t replicate onto yourself';
    RETURN -1;
  END IF;

  insert into replication.nodes_skip(num) values(remote_num);

  RETURN 0;
 END;
$$
LANGUAGE plpgsql strict;

/** FUNCTION replication.stop(remote_num int)
Stops the replication with a neighbour node. This function is called by replication.pl.

@param remote_num neighbour node number

@return 0 upon success, -1 upon error

*/
CREATE OR REPLACE FUNCTION replication.stop(remote_num int)
RETURNS int
AS $$
 DECLARE
  n int;
 BEGIN
  SELECT INTO n val::int from replication.config where key='node_num';
  IF NOT FOUND THEN
    RAISE EXCEPTION 'Replication has not been set up on this node.';
    RETURN -1;
  END IF;
  IF n = remote_num THEN
    RAISE EXCEPTION 'You can''t replicate onto yourself';
    RETURN -1;
  END IF;

  delete from replication.nodes_skip where num=remote_num;

  RETURN 0;
 END;
$$
LANGUAGE plpgsql strict;

/** FUNCTION replication.truncate(tablename_ text)
Truncates a table.

@param tablename_ name of table to truncate. You may have to provide the schema name if the schema is not in the default search path.

@return 0 upon success, -1 upon error

*/
CREATE OR REPLACE FUNCTION replication.truncate(tablename_ text)
RETURNS int
AS $$
  SELECT replication.ddl('truncate ' || $1);
$$
LANGUAGE 'sql' strict;

/** FUNCTION lo_dump(id oid)
Converts a large object to text.

@param lo large object OID

@return text

*/
CREATE OR REPLACE FUNCTION lo_dump(id oid)
RETURNS text
AS $$
 DECLARE
  fd        integer;
  size      integer;
  txt       text;
  INV_READ  constant integer := x'40000'::integer; -- from libpq-fs.h
  SEEK_SET  constant integer := 0;
  SEEK_END  constant integer := 2;
 BEGIN
  fd   := lo_open(id, INV_READ);
  size := lo_lseek(fd, 0, SEEK_END);
  PERFORM lo_lseek(fd, 0, SEEK_SET);
  txt := loread(fd, size);
  PERFORM lo_close(fd);
  RETURN txt;
 END;
$$
LANGUAGE plpgsql stable strict;
COMMENT ON FUNCTION replication.nodes_insert() IS E'Trigger function which writes INSERT rows into the replication.log
table for all replicated tables when a new node is added. The function
is called by an insert trigger on table replication.nodes.';
COMMENT ON FUNCTION replication.newevent(tablename_ text, action_ text, nr_ int8, lo_ int8) IS E'Inserts a new event into replication.log for all neighbour nodes.

@param tablename_ table name which contains row

@param action_ event type. See TABLE replication.log for valid values.

@param nr_ row identifier or NULL

@param lo_ large object OID or NULL';
COMMENT ON FUNCTION replication.tableevent() IS E'Trigger function on replicated table without large object (lo) column which creates entries in replication.log.';
COMMENT ON FUNCTION replication.loevent() IS E'Trigger function on replicated table with large object (lo) column which creates entries in replication.log.';
COMMENT ON FUNCTION replication.setseq(tablename text) IS E'Sets the "nr" sequence on "tablename" to a new value which will allow the table to be used in replication.

@return -1 upon error, 1 upon success';
COMMENT ON FUNCTION replication.setallseq() IS E'Sets "nr" sequences for all replicated tables.

@return 0 upon success';
COMMENT ON FUNCTION replication._register(tablename text) IS E'Internal function to add a table to replication.

@return tablename with schema upon success, NULL upon error';
COMMENT ON FUNCTION replication.register(tablename text) IS E'Register a table for replication.

@param tablename name of the table. You may have to provide the schema name if the table is not in the ''public'' schema.

@return tablename (full qualified) upon success, NULL upon error';
COMMENT ON FUNCTION replication.registerlo(tablename text) IS E'Register a table with "lo" column to manage large objects for replication.

@param tablename name of the table. You may have to provide the schema name if the table is not in the ''public'' schema.

@return tablename (full qualified) upon success, NULL upon error';
COMMENT ON FUNCTION replication.node_num(num_ int, d text) IS E'Configure this database for replication.

@param num_ node number of this database

@param d description string of this node

@return num_ upon success, -1 upon error';
COMMENT ON FUNCTION replication.drop_node(n int) IS E'Drop a node from replication.

@param n node number.

@return n upon success, -1 upon error.';
COMMENT ON FUNCTION md5(id oid) IS E'Calculates the MD5 sum of a large object. If no large object is found
the MD5 sum of the string value of id is calculated.

@param id OID of large object

@return MD5 sum';
COMMENT ON FUNCTION replication.ddl(t text) IS E'Executes and replicates DDL commands.

@param t string containing DDL

@return 0 upon success, -1 upon error';
COMMENT ON FUNCTION replication.start(remote_num int) IS E'Starts a replication with a neighbour node. This function is called by replication.pl.

@param remote_num neighbour node number

@return 0 upon success, -1 upon error';
COMMENT ON FUNCTION replication.stop(remote_num int) IS E'Stops the replication with a neighbour node. This function is called by replication.pl.

@param remote_num neighbour node number

@return 0 upon success, -1 upon error';
COMMENT ON FUNCTION replication.truncate(tablename_ text) IS E'Truncates a table.

@param tablename_ name of table to truncate. You may have to provide the schema name if the schema is not in the default search path.

@return 0 upon success, -1 upon error';
COMMENT ON FUNCTION lo_dump(id oid) IS E'Converts a large object to text.

@param lo large object OID

@return text';
-- $Header: /code/pg_trompe/tables.sql,v 1.1 2009/01/09 23:14:49 doj Exp $

/** TABLE replication.config
Contains configuration "variables" for replication process.

@column key variable name

@column val variable value

@column description variable description

*/
CREATE TABLE replication.config (
  key	text not null primary key,
  val	text,
  description	text
);

insert into replication.config(key,val,description) values('maxnodes', '10', 'Maximum number of nodes.');

/** TABLE replication.tables
Contains all table names which are replicated.

@column nr primary key containing unique integer number of the row

@column name name of the table

@column lo if true the table contains an "lo" column referencing large object OIDs.

*/
CREATE TABLE replication.tables (
  nr	serial not null primary key,
  name	text not null unique,
  lo	boolean not null default FALSE
);

/** TABLE replication.nodes
Contains information on the replication neighbour nodes.

@column num node number

@column description node information string

@column added when node was added to replication

@column lastsync of last replication run

*/
CREATE TABLE replication.nodes (
  num		int not null primary key,
  description	text,
  added		timestamp not null default now(),
  lastsync	timestamp
);

CREATE TRIGGER nodes_insert_trig
AFTER INSERT
ON replication.nodes
FOR EACH ROW EXECUTE PROCEDURE replication.nodes_insert();

/** TABLE replication.nodes_skip

Contains node numbers of nodes which should be skipped by the
replication triggers. This table contains our own node number and the
remote node number during replication.

@column num node number of a node from table replication.nodes

*/
CREATE TABLE replication.nodes_skip (
  num		int not null primary key
);

/** TABLE replication.log

Contains events on replicated tables which have not been replicated to neighbour nodes.

@column id sequential integers to keep track of event ordering

@column node node number to which event must be replicated

@column tablename name of the table

@column action event type. Allowed values are 'INSERT', 'UPDATE', 'DELETE', 'DDL'.

@column nr unique row-identifying number in "tablename". Is NULL for 'DDL' action.

@column lo OID of affected large object. Can be NULL if no large object is involved in this event.

*/
CREATE TABLE replication.log (
  id		serial8 primary key,
  node		int  not null references replication.nodes on update cascade on delete cascade deferrable,
  tablename	text not null,
  action	text not null check(action IN ('INSERT', 'UPDATE', 'DELETE', 'DDL')),
  nr		int8,
  lo		int8
);

/** VIEW replication.nodestats
Shows the amount of events not replicated to neighbour nodes.
*/
CREATE OR REPLACE VIEW replication.nodestats AS
select node, count(*) as pending_updates, (select lastsync from replication.nodes where num=node) as lastsync
from replication.log
group by node;
COMMENT ON COLUMN replication.config.key IS E'variable name';
COMMENT ON COLUMN replication.config.val IS E'variable value';
COMMENT ON COLUMN replication.config.description IS E'variable description';
COMMENT ON TABLE replication.config IS E'Contains configuration "variables" for replication process.';
COMMENT ON COLUMN replication.tables.nr IS E'primary key containing unique integer number of the row';
COMMENT ON COLUMN replication.tables.name IS E'name of the table';
COMMENT ON COLUMN replication.tables.lo IS E'if true the table contains an "lo" column referencing large object OIDs.';
COMMENT ON TABLE replication.tables IS E'Contains all table names which are replicated.';
COMMENT ON COLUMN replication.nodes.num IS E'node number';
COMMENT ON COLUMN replication.nodes.description IS E'node information string';
COMMENT ON COLUMN replication.nodes.added IS E'when node was added to replication';
COMMENT ON COLUMN replication.nodes.lastsync IS E'of last replication run';
COMMENT ON TABLE replication.nodes IS E'Contains information on the replication neighbour nodes.';
COMMENT ON COLUMN replication.nodes_skip.num IS E'node number of a node from table replication.nodes';
COMMENT ON TABLE replication.nodes_skip IS E'Contains node numbers of nodes which should be skipped by the
replication triggers. This table contains our own node number and the
remote node number during replication.';
COMMENT ON COLUMN replication.log.id IS E'sequential integers to keep track of event ordering';
COMMENT ON COLUMN replication.log.node IS E'node number to which event must be replicated';
COMMENT ON COLUMN replication.log.tablename IS E'name of the table';
COMMENT ON COLUMN replication.log.action IS E'event type. Allowed values are ''INSERT'', ''UPDATE'', ''DELETE'', ''DDL''.';
COMMENT ON COLUMN replication.log.nr IS E'unique row-identifying number in "tablename". Is NULL for ''DDL'' action.';
COMMENT ON COLUMN replication.log.lo IS E'OID of affected large object. Can be NULL if no large object is involved in this event.';
COMMENT ON TABLE replication.log IS E'Contains events on replicated tables which have not been replicated to neighbour nodes.';
COMMENT ON VIEW replication.nodestats IS E'Shows the amount of events not replicated to neighbour nodes.';
