-- $Header: /code/pg_trompe/tables.sql,v 1.2 2009/06/26 05:46:56 doj Exp $

/** TABLE replication.config
Contains configuration "variables" for replication process.

@column key variable name

@column val variable value

@column description variable description

*/
CREATE TABLE replication.config (
  key	text not null primary key,
  val	text,
  description	text
);

insert into replication.config(key,val,description) values
('maxnodes', '10', 'Maximum number of nodes.'),
('pg_trompe version', '8.3.1', 'Software version of pg_trompe used on this node');

/** TABLE replication.tables
Contains all table names which are replicated.

@column nr primary key containing unique integer number of the row

@column name name of the table

@column lo if true the table contains an "lo" column referencing large object OIDs.

*/
CREATE TABLE replication.tables (
  nr	serial not null primary key,
  name	text not null unique,
  lo	boolean not null default FALSE
);

/** TABLE replication.nodes
Contains information on the replication neighbour nodes.

@column num node number

@column description node information string

@column added when node was added to replication

@column lastsync of last replication run

*/
CREATE TABLE replication.nodes (
  num		int not null primary key,
  description	text,
  added		timestamp not null default now(),
  lastsync	timestamp
);

CREATE TRIGGER nodes_insert_trig
AFTER INSERT
ON replication.nodes
FOR EACH ROW EXECUTE PROCEDURE replication.nodes_insert();

/** TABLE replication.nodes_skip

Contains node numbers of nodes which should be skipped by the
replication triggers. This table contains our own node number and the
remote node number during replication.

@column num node number of a node from table replication.nodes

*/
CREATE TABLE replication.nodes_skip (
  num		int not null primary key
);

/** TABLE replication.log

Contains events on replicated tables which have not been replicated to neighbour nodes.

@column id sequential integers to keep track of event ordering

@column node node number to which event must be replicated

@column tablename name of the table

@column action event type. Allowed values are 'INSERT', 'UPDATE', 'DELETE', 'DDL'.

@column nr unique row-identifying number in "tablename". Is NULL for 'DDL' action.

@column lo OID of affected large object. Can be NULL if no large object is involved in this event.

*/
CREATE TABLE replication.log (
  id		serial8 primary key,
  node		int  not null references replication.nodes on update cascade on delete cascade deferrable,
  tablename	text not null,
  action	text not null check(action IN ('INSERT', 'UPDATE', 'DELETE', 'DDL')),
  nr		int8,
  lo		int8
);

/** VIEW replication.nodestats
Shows the amount of events not replicated to neighbour nodes.
*/
CREATE OR REPLACE VIEW replication.nodestats AS
select node, count(*) as pending_updates, (select lastsync from replication.nodes where num=node) as lastsync
from replication.log
group by node;
